@@ -1,22 +1,24 @@
-#' Generalized Tensor Regression
+#' Supervised Tensor Decomposition with Interactive Side Information
+#'
+#' Supervised tensor decomposition with interactive side information on multiple modes. Main function in the package. The function takes a response tensor, multiple side information matrices,
+#' and a desired Tucker rank as input. The output is a rank-constrained M-estimate of the core tensor and factor matrices.
 #'
-#' Tensor-response regression given covariates on multiple modes. Main function in the package. The function takes a response tensor, multiple covariate matrices, and a desired Tucker rank as input. The output is a constrained
-#' MLE for the coefficient tensor.
 #'
 #' @param tsr    response tensor with 3 modes
-#' @param X_covar1    covariate on first mode
-#' @param X_covar2    covariate on second mode
-#' @param X_covar3    covariate on third mode
-#' @param core_shape  the Tucker rank of the regression coefficients
+#' @param X_covar1    side information on first mode
+#' @param X_covar2    side information on second mode
+#' @param X_covar3    side information on third mode
+#' @param core_shape  the Tucker rank of the tensor decomposition
 #' @param Nsim        max number of iterations if update does not convergence
 #' @param cons        the constraint method, "non" for without constraint, "vanilla" for global scale down at each iteration, "penalty" for adding log-barrier penalty to object function
 #' @param lambda      penalty coefficient for "penalty" constraint
 #' @param alpha       max norm constraint on linear predictor
 #' @param solver      solver for solving object function when using "penalty" constraint, see "details"
 #' @param dist        distribution of the response tensor, see "details"
+#' @param traj_long   if "TRUE", set the minimal iteration number to 8; if "FALSE", set the minimal iteration number to 0
 #' @return     a list containing the following:
 #'
-#'                  \code{W} {a list of orthogonal coefficient matrices - one for each mode, with the number of columns given by \code{core_shape}}
+#'                  \code{W} {a list of orthogonal factor matrices - one for each mode, with the number of columns given by \code{core_shape}}
 #'
 #'                  \code{G}  {an array, core tensor with the size specified by \code{core_shape}}
 #'
@@ -38,10 +40,9 @@
 #'
 #'            \code{dist} specifies three distributions of response tensor: binary, poisson and normal distribution.
 #'
-#' @author Z. Xu \email{zxu444@wisc.edu}  and J. Hu \email{jhu267@wisc.edu}
-#' @references Z. Xu, J. Hu, and M. Wang, "Generalized Tensor Regression with Covariates on Multiple Modes". 2019. <arXiv:1910.09499>. URL: https://arxiv.org/abs/1910.09499
+#'
 #' @export
-#' @examples 
+#' @examples
 #' seed = 34
 #' dist = 'binary'
 #' data=sim_data(seed, whole_shape = c(20,20,20), core_shape=c(3,3,3),
@@ -49,7 +50,7 @@
 #' re = tensor_regress(data$tsr[[1]],data$X_covar1,data$X_covar2,data$X_covar3,
 #' core_shape=c(3,3,3),Nsim=10, cons = 'non', dist = dist)
 
-tensor_regress = function(tsr,X_covar1 = NULL, X_covar2 = NULL,X_covar3 = NULL, core_shape, Nsim=20, cons = c("non","vanilla","penalty"), lambda = 0.1, alpha = 1, solver ="CG",dist = c("binary", "poisson","normal")){
+tensor_regress = function(tsr,X_covar1 = NULL, X_covar2 = NULL,X_covar3 = NULL, core_shape, Nsim=20, cons = c("non","vanilla","penalty"), lambda = 0.1, alpha = 1, solver ="CG",dist = c("binary", "poisson","normal"),traj_long=FALSE){
 
 
   tsr = as.tensor(tsr)
@@ -70,16 +71,19 @@ tensor_regress = function(tsr,X_covar1 = NULL, X_covar2 = NULL,X_covar3 = NULL,
   if(dist=="binary"){
     tsr.transform=as.tensor(2*tsr@data-1)
   }else if(dist=="poisson"){
-    tsr.transform=as.tensor(log(tsr@data+0.1))###?? new initilization
+    tsr.transform=as.tensor(log(tsr@data+0.1))
   }else if (dist=="normal"){
     tsr.transform=tsr
   }
 
   C_ts=ttl(tsr.transform,list(ginv(X_covar1),ginv(X_covar2),ginv(X_covar3)),ms=c(1,2,3))
 
-  tckr = tucker(C_ts, ranks = core_shape)
-  W1 = tckr$U[[1]] ; W2 = tckr$U[[2]] ; W3 = tckr$U[[3]] ## tucker factors
-  G = tckr$Z
+  #tckr = tucker(C_ts, ranks = core_shape)
+  #W1 = tckr$U[[1]] ; W2 = tckr$U[[2]] ; W3 = tckr$U[[3]] ## tucker factors
+  #G = tckr$Z
+  W1=randortho(p1)[,1:core_shape[1]];W2=randortho(p2)[,1:core_shape[2]];W3=randortho(p3)[,1:core_shape[3]]
+  G=ttl(C_ts,list(t(W1),t(W2),t(W3)),ms=1:3)
+
   A = X_covar1%*%W1
   B = X_covar2%*%W2
   C = X_covar3%*%W3
@@ -154,16 +158,22 @@ tensor_regress = function(tsr,X_covar1 = NULL, X_covar2 = NULL,X_covar3 = NULL,
 
     #########-----------------------------------------------
     ###  obtain core tensor under constraint
-    core=update_core(tsr,G,A,B,C,core_shape,cons,lambda,alpha,solver,dist)
+ core=update_core(tsr,G,A,B,C,core_shape,cons,lambda,alpha,solver,dist)
     G=core$G
     lglk=c(lglk,core$lglk)
     violate=c(violate,core$violate)
 
     #print("G Done------------------")
 
+
     message(paste(n,"-th  iteration -- when dimension is",d1,d2,d3,"- rank is ",r1,r2,r3," -----------------"))
     #print(paste(n,"-th  iteration"))
 
+    if((traj_long==T)&(n < 8)){
+     n = n+1
+     next
+    }
+
     if ((tail(lglk,1)-lglk0)/abs(lglk0)<= 0.0001 & tail(lglk,1)>= lglk0 ){
       message(paste(n,"-th iteration: convergence"))
       break
@@ -185,13 +195,13 @@ tensor_regress = function(tsr,X_covar1 = NULL, X_covar2 = NULL,X_covar3 = NULL,
 
 
 
-#' Simulation of tensor regression models
+#' Simulation of supervised tensor decomposition models
 #'
-#' Generate response tensors with multiple covariates under different simulation models, specifically for tensors with 3 modes
+#' Generate tensor data with multiple side information matrices under different simulation models, specifically for tensors with 3 modes
 #' @param seed         a random seed for generating data
 #' @param whole_shape  a vector containing dimension of the tensor
-#' @param core_shape   a vector containing Tucker rank of the coefficient tensor
-#' @param p            a vector containing numbers of covariates on each mode, see "details"
+#' @param core_shape   a vector containing Tucker rank of the tensor decomposition
+#' @param p            a vector containing numbers of side information on each mode, see "details"
 #' @param dist         distribution of response tensor, see "details"
 #' @param dup          number of simulated tensors from the same linear predictor
 #' @param signal       a scalar controlling the max norm of the linear predictor
@@ -200,13 +210,13 @@ tensor_regress = function(tsr,X_covar1 = NULL, X_covar2 = NULL,X_covar3 = NULL,
 #'
 #' \code{tsr} {a list of simulated tensors, with the number of replicates specified by \code{dup}}
 #'
-#' \code{X_covar1}  {a matrix, covariate on first mode}
+#' \code{X_covar1}  {a matrix, side information on first mode}
 #'
-#' \code{X_covar2}  {a matrix, covariate on second mode}
+#' \code{X_covar2}  {a matrix, side information on second mode}
 #'
-#' \code{X_covar3}  {a matrix, covariate on third mode}
+#' \code{X_covar3}  {a matrix, side information on third mode}
 #'
-#' \code{W} {a list of orthogonal coefficient matrices - one for each mode, with the number of columns given by \code{core_shape}}
+#' \code{W} {a list of orthogonal factor matrices - one for each mode, with the number of columns given by \code{core_shape}}
 #'
 #' \code{G}  {an array, core tensor with size specified by \code{core_shape}}
 #'
@@ -222,7 +232,7 @@ tensor_regress = function(tsr,X_covar1 = NULL, X_covar2 = NULL,X_covar3 = NULL,
 #'
 #'
 #' @export
-#' @examples 
+#' @examples
 #' seed = 34
 #' dist = 'binary'
 #' data=sim_data(seed, whole_shape = c(20,20,20), core_shape=c(3,3,3),
@@ -233,69 +243,69 @@ tensor_regress = function(tsr,X_covar1 = NULL, X_covar2 = NULL,X_covar3 = NULL,
 #####---- This is the function used for generating data through different distribution
 #         of core tensor in  semi-supervised setting
 ## p is the dimension of the covaraite. p = 0 represents the case without covaraites
-sim_data = function(seed, whole_shape = c(20,20,20), core_shape = c(3,3,3),p=c(3,3,0),dist, dup, signal,block=rep(FALSE,3)){
-  
+sim_data = function(seed=NA, whole_shape = c(20,20,20), core_shape = c(3,3,3),p=c(3,3,0),dist, dup, signal,block=rep(FALSE,3)){
+
   d1 = whole_shape[1] ; d2 = whole_shape[2] ; d3 = whole_shape[3]
   r1 = core_shape[1] ; r2 = core_shape[2] ; r3 = core_shape[3]
   p1 = p[1]; p2 = p[2]; p3 = p[3];
-  
+
   #### warning for r should larger than 0
   if(r1<=0 | r2 <= 0|r3<= 0){
     warning("the rank of coefficient tensor should be larger than 0",immediate. = T)
     return()
   }
-  
+
   if(p1 > d1|p2 > d2|p3 > d3){
     warning("the number of covariates at each mode should be no larger than the dimension of the tensor",immediate. = T)
   }
-  
+
   #### warning for p should larger than r
   if((p1<r1 & p1>0)|(p2<r2 & p2>0)|(p3<r3 & p3>0)){
     warning("the rank of coefficient tensor should be no larger than the number of covariates",immediate. = T)
   }
-  
-  
-  
+
+
+
   ####-------- generate data
-  set.seed(seed)  # 24 # 37  #  347
+  if(is.na(seed)==FALSE) set.seed(seed)
   X_covar1 = X_covar2 = X_covar3 = NULL
-  
+
   if(p1<=0){
     X_covar1=diag(1,d1)
     p1=d1
   }else{
     X_covar1 = matrix(rnorm(d1*p1,mean = 0, sd = 1/sqrt(d1)),d1,p1)
   }
-  
+
   if(p2<=0){
     X_covar2=diag(1,d2)
     p2=d2
   }else{
     X_covar2 = matrix(rnorm(d2*p2,mean = 0, sd =1/sqrt(d2)),d2,p2)
   }
-  
-  
+
+
   if(p3<=0){
     X_covar3=diag(1,d3)
     p3=d3
   }else{
     X_covar3 = matrix(rnorm(d3*p3,mean = 0, sd = 1/sqrt(d3)),d3,p3)
   }
-  
+
   #### if use block, p and r should larger than 1 of smaller than 0
   if(block[1] == T & (p1 ==1|r1 == 1)){
     warning("number of groups should be larger than 1",immediate. = T)
   }
-  
+
   if(block[2] == T & (p2 ==1|r2 == 1)){
     warning("number of groups should be larger than 1",immediate. = T)
   }
-  
+
   if(block[3] == T & (p3 ==1|r3 == 1)){
     warning("number of groups should be larger than 1",immediate. = T)
   }
-  
-  
+
+
   if (block[1]==TRUE){
     b1=sort(sample(1:r1,p1,replace=TRUE))
     if(length(unique(b1) )== 1){
@@ -304,9 +314,9 @@ sim_data = function(seed, whole_shape = c(20,20,20), core_shape = c(3,3,3),p=c(3
     W1=model.matrix(~-1+as.factor(b1))
     r1=dim(W1)[2]
   }else W1 =as.matrix(randortho(p1)[,1:r1])
-  
+
   A = X_covar1%*%W1 ## factor matrix
-  
+
   if (block[2]==TRUE){
     b2=sort(sample(1:r2,p2,replace=TRUE))
     if(length(unique(b2)) == 1){
@@ -316,7 +326,7 @@ sim_data = function(seed, whole_shape = c(20,20,20), core_shape = c(3,3,3),p=c(3
     r2=dim(W2)[2]
   }else W2 = as.matrix(randortho(p2)[,1:r2])
   B = X_covar2%*%W2 ## factor matrix
-  
+
   if (block[3]==TRUE){
     b3=sort(sample(1:r3,p3,replace=TRUE))
     if(length(unique(b3)) == 1){
@@ -326,19 +336,19 @@ sim_data = function(seed, whole_shape = c(20,20,20), core_shape = c(3,3,3),p=c(3
     r3=dim(W3)[2]
   }else W3 = as.matrix(randortho(p3)[,1:r3])
   C= X_covar3%*%W3 ## factor matrix
-  
-  
+
+
   ### G: core tensor
   G = as.tensor(array(runif(r1*r2*r3,min=-1,max=1),dim = c(r1,r2,r3)))
-  
-  
+
+
   ### U: linear predictor
   U = ttl(G,list(A,B,C),ms = c(1,2,3))@data
   G=G/max(abs(U))*signal ## rescale subject to entrywise constraint
   U=U/max(abs(U))*signal
-  
+
   C_ts=ttl(G,list(W1,W2,W3),ms = c(1,2,3))@data ## coefficient
-  
+
   ### tsr:binary tensor
   if(dist=="binary"){
     tsr = lapply(seq(dup), function(x) array(rbinom(d1*d2*d3,1,prob = as.vector( 1/(1 + exp(-U)))),dim = c(d1,d2,d3)))}
@@ -346,8 +356,8 @@ sim_data = function(seed, whole_shape = c(20,20,20), core_shape = c(3,3,3),p=c(3
     tsr = lapply(seq(dup), function(x) array(rnorm(d1*d2*d3,U),dim = c(d1,d2,d3)))}#sd = 1
   else if (dist=="poisson"){
     tsr = lapply(seq(dup), function(x) array(rpois(d1*d2*d3,exp(U)),dim = c(d1,d2,d3)))}
-  
-  
+
+
   return(list(tsr = tsr,X_covar1 = X_covar1, X_covar2 = X_covar2,X_covar3 = X_covar3,
               W = list(W1 = W1,W2 = W2,W3 = W3), G=G@data, U=U,C_ts=C_ts))
 }
@@ -357,12 +367,12 @@ sim_data = function(seed, whole_shape = c(20,20,20), core_shape = c(3,3,3),p=c(3
 
 #' Rank selection
 #'
-#' Estimate the Tucker rank of coefficient tensor based on BIC criterion. The choice of BIC
+#' Estimate the Tucker rank of tensor decomposition based on BIC criterion. The choice of BIC
 #'  aims to balance between the goodness-of-fit for the data and the degree of freedom in the population model.
 #' @param tsr    response tensor with 3 modes
-#' @param X_covar1    covariate on first mode
-#' @param X_covar2    covariate on second mode
-#' @param X_covar3    covariate on third mode
+#' @param X_covar1    side information on first mode
+#' @param X_covar2    side information on second mode
+#' @param X_covar3    side information on third mode
 #' @param rank_range  a matrix containing rank candidates on each row
 #' @param Nsim        max number of iterations if update does not convergence
 #' @param cons        the constraint method, "non" for without constraint, "vanilla" for global scale down at each iteration,
@@ -379,21 +389,21 @@ sim_data = function(seed, whole_shape = c(20,20,20), core_shape = c(3,3,3),p=c(3
 #'                    \code{result}  a matrix containing rank candidate and its loglikelihood and BIC on each row
 
 #' @details    For rank selection, recommend using non-constraint version.
-#'             
+#'
 #'            Constraint \code{penalty} adds log-barrier regularizer to
 #'            general object function (negative log-likelihood). The main function uses solver in function "optim" to
 #'            solve the objective function. The "solver" passes to the argument "method" in function "optim".
-#'            
+#'
 #'             \code{dist} specifies three distributions of response tensor: binary, poisson and normal distributions.
 #'
 #'
 #' @export
-#' @examples 
+#' @examples
 #' seed=24
 #' dist='binary'
 #' data=sim_data(seed, whole_shape = c(20,20,20),
 #' core_shape=c(3,3,3),p=c(5,5,5),dist=dist, dup=5, signal=4)
-#' rank_range = rbind(c(3,3,3),c(3,3,2),c(3,2,2),c(2,2,2),c(3,2,3)) 
+#' rank_range = rbind(c(3,3,3),c(3,3,2),c(3,2,2),c(2,2,2),c(3,2,3))
 #' re = sele_rank(data$tsr[[1]],data$X_covar1,data$X_covar2,data$X_covar3,
 #'  rank_range = rank_range,Nsim=10,cons = 'non',dist = dist)
 
@@ -403,25 +413,25 @@ sim_data = function(seed, whole_shape = c(20,20,20), core_shape = c(3,3,3),p=c(3
 
 
 sele_rank = function(tsr, X_covar1 = NULL, X_covar2 = NULL, X_covar3 = NULL,rank_range,Nsim=10,cons = 'non', lambda = 0.1, alpha = 1, solver ='CG',dist){
-  whole_shape=dim(tsr)                      
+  whole_shape=dim(tsr)
   p=rep(0,3)
   if(is.null(X_covar1)) p[1]=whole_shape[1] else p[1]=dim(X_covar1)[2]
   if(is.null(X_covar2)) p[2]=whole_shape[2] else p[2]=dim(X_covar2)[2]
   if(is.null(X_covar3)) p[3]=whole_shape[3] else p[3]=dim(X_covar3)[2]
-  
-  
+
+
   rank_matrix=rank_range
   rank=as.matrix(rank_range)
-  
+
   whole_shape = dim(tsr)
   rank = lapply(1:dim(rank)[1], function(x) rank[x,]) ## turn rank to a list
   upp = lapply(rank, FUN= tensor_regress,tsr = tsr,X_covar1 = X_covar1,X_covar2 = X_covar2,X_covar3 = X_covar3, Nsim = Nsim, cons = cons,lambda = lambda, alpha = alpha, solver = solver,dist=dist)
-  
+
   lglk= unlist(lapply(seq(length(upp)), function(x) tail(upp[[x]]$lglk,1)))
   BIC = unlist(lapply(seq(length(rank)), function(x) (prod(rank[[x]]) + sum((p-rank[[x]])*rank[[x]])) * log(prod(whole_shape))))
   BIC = -2*lglk + BIC
   rank_matrix=cbind(rank_matrix,lglk,BIC)
-  
+
   return(list(rank = rank[[which(BIC == min(BIC))]],result=rank_matrix))
 }
 
